#! /bin/csh -f
#
#	script for autoindexing mosflm spot output with dps_index
#	without the need for a GUI
#
########
# 
# use the most recently-created .spt file by default
set spotfile  = `ls -1rt *.spt | tail -1`
set outmatrix = dps.mat
set tempfile  = dps_temp

# camera settings (not available in *.spt file)
set wavelength  = ""
set distance    = ""
set two_theta    = 0
set beam_center = ""

# crystal parameters (overridden by keywords in script)
set CELL        = ""
set SG          = ""
set max_cell    = 300
set hiRES       = 1
set loRES       = 1000

goto Setup
Help:
############################################################
cat << EOF
usage: dps_index.com mosflm.spt 1.0A 100mm P212121
  or
usage: dps_index.com mosflm.spt mosflm.com P212121

where:
mosflm.spt  - the "spot" file (picked with mosflm)
1.0A        - the x-ray wavelength
100mm       - the xtf distance
mosflm.com  - a mosflm script containing CELL, DIST, WAVE, etc., 
P212121     - space group to select (optional)
3-20A	    - resolution range to use (optional)
50sig	    - I/sigma cutoff to use (default: 0)
300	    - maximum cell edge (default: $max_cell)

EOF
exit 9
Return_from_Setup:

if("$wavelength" == "") goto Help
if("$distance" == "") goto Help

############################################################
# retrieve useful parameters from the mosflm spot file
echo "$0 $*"
echo "using ${spotfile} ($frames images)"
set rotation    = `nawk 'NR==1{rot=$5} NR==2{print (360-rot+180*$1)%360;exit}' $spotfile`
set ysign       = `nawk 'NR==2{print 1-($1*2);exit}' $spotfile`
set raster      = `nawk '{print $3;exit}' $spotfile`
set yscale      = `nawk '{print $4;exit}' $spotfile`
set beam_center = `nawk 'NR==3 && NF==2{print;exit}' $spotfile`
set beamstop    = `echo $beam_center $raster $yscale | nawk '{printf "r %d s %d", $1/$3,$2*$4/$3}'`
set sscale      = `echo $ysign $yscale | nawk '{print $1/$2}'`
set mode        = automatic

# are these important?
set phi0        = `nawk 'NR==4{printf "%.1f", $4-$3}' ${spotfile}`
set phiend      = `nawk 'NR==4{printf "%.1f", $4+$3}' ${spotfile}`


# report values input into dps_index
echo "wavelength:  $wavelength"
echo "beam center: $beamstop"
echo "distance:    $distance"
echo "2theta:      $two_theta"
echo "sscale:      $sscale"
echo "rotation:    $rotation"
echo "resolution:  $loRES $hiRES"
echo "sigma cut:   $cutoff"
if("$CELL" != "") then
    echo "Cell: $CELL"
    set CELL = "target_cell $CELL"
    set mode = "target_cell"
else
    echo "Cell:      < $max_cell"
endif
if("$SG" != "") then
    echo "Space Group: $SG"
else
    if("$wrongSGs" != "") echo "Space Group: anything but $wrongSGs"
endif
echo ""

# convert the mosflm spot file into a DPS peaks file
cat $spotfile |\
nawk -v cutoff=$cutoff 'BEGIN{PI=2*atan2(1,0); print "DPS-PF A2.0"}\
  NR==1{xwidth = $3; ywidth = $3/$4;}\
  NF>5 && $1!="-99.00" && $1!="-999.00" && $6+0>0{\
    if($5/$6<cutoff) next; \
    print $1/xwidth, $2/ywidth,$4*PI/180}' |\
cat >! peaks.file

set temp = `head -1 $spotfile`
if("$temp" =~ DPS-PF*) then
    cp $spotfile peaks.file
endif

# now do DPS indexing
foreach round ( 1 2 3 )
dps_index << EOF | tee ${tempfile}.log
mode $mode
$CELL
$target_sym
resolution $loRES $hiRES
cell $max_cell
wavelength $wavelength
rastersize $raster
distance $distance
twotheta $two_theta
beamstop $beamstop
film_rotation $rotation
sscale $sscale
PEAKS peaks.file $phi0 $phiend
omfile auto.om
EOF
if(! -e PEAKS_WITH_PHIS) then
    exit 9
endif
# keep re-cycling peak file
mv PEAKS_WITH_PHIS peaks.file

set RMS = `nawk '/RMS/{printf "%d", $NF}' ${tempfile}.log`
rm -f ${tempfile}.log
if($RMS < 3) break
end

# score different lattice possibilities
dps_bravais << EOF | tee ${tempfile}.log
mode bravais
omfile auto.om
EOF

# organize possibilities in terms of space groups
cat ${tempfile}.log |\
nawk -v library=$CLIBD/symop.lib '/^Lattice:/{\
	solution=$2; crys_sys=$4; score=$6; source="|";\
	sys = substr($4,1,1); lattice = substr($4,2); \
	getline;getline;getline;getline; cell=substr($0,10);\
	if(sys=="a") sys = "TRICL"; \
	if(sys=="m") sys = "MONOC"; \
	if(sys=="o") sys = "ORTHO"; \
	if(sys=="t") sys = "TETRA"; \
	if(sys=="h") sys = "TRIGO"; \
	if(sys=="c") sys = "CUBIC"; \
	while(getline < library){ \
	    if($5 ~ /^PG/ && ! /m/ && ! /bar/ && $1 < 500){\
		SYS=substr($6,1,5)\
		if(substr($4,1,1)==lattice && (SYS==sys || (SYS=="HEXAG" && sys=="TRIGO"))){\
		    print $1, substr($5,3), $4, crys_sys, $6, solution, score, cell, source;\
		}\
	    }\
	}; close(library);}' |\
sort +0nr -1 +6n -7  |\
nawk '! seen[$3]{print; seen[$3]=1}' |\
nawk '{a=$8; b=$9; c=$10; A=$11; B=$12; G=$13;latt=$5}\
    $5 == "MONOCLINIC" { A=90; ; G=90}\
    $5 == "ORTHORHOMBIC" {A=90; B=90; G=90}\
    $5 == "TETRAGONAL" || $5 == "TRIGONAL" || $5 == "HEXAGONAL" {\
	a=b=(a+b)/2; A=90; B=90; G=120;}\
    $5 == "TETRAGONAL" {G=90}\
    $5 == "CUBIC" {a=b=c=(a+b+c)/3; A=90; B=90; G=90}\
    {print $0, a, b, c, A, B, G}' |\
cat >! ${tempfile}.sg
# format: SG_num PG SG crys_sys lattice solution penalty cell source newcell

# clear undesirable SG choices from the list of possibilities
echo " $wrongSGs LIST " |\
nawk 'BEGIN{RS=" "} NF==1' |\
cat - ${tempfile}.sg |\
nawk 'NF==1{bad[$1]=1} \
           list && ! bad[$3] && ! bad[$4] && ! bad[$5] {print}\
         /LIST/{list=1}' |\
cat >! ${tempfile}filtered
mv ${tempfile}filtered ${tempfile}.sg
# format: SG_num PG SG crys_sys lattice solution penalty cell source newcell

# automatically pick space group with good fit
if("$SG" == "") then
    # choose one...
    set solution = `nawk '$7<1{print $6; exit}' ${tempfile}.sg`
    if("$solution" == "") then
	set solution = `nawk '{print $6}' ${tempfile}.sg | tail -1`
    endif
    set SG = `nawk -v solution=$solution '$6==solution{print $3}' ${tempfile}.sg | tail -1`
    echo "selected $SG as a representative of the point group"
endif
set solution = `nawk -v SG=$SG '$3==SG{print $6;exit}' ${tempfile}.sg`

bravais_to_mosflm $solution ${tempfile}.log /dev/null ${outmatrix} 0.0 ${wavelength} no_update

condense_bravais ${tempfile}.log bravais.sum
set CELL = `nawk 'NF==6{print;exit}' ${outmatrix}`
if($#CELL != 6) then
    # do it ourselves...
    cat ${tempfile}.log |\
    nawk -v solution=$solution '/Lattice/ && $2==solution{\
	getline; mid=$0; getline; last=$0; getline; \
	print $0; print mid; print last; \
	getline; print}' |\
    nawk -v wave=$wavelength 'NF==3{printf "%12.8f%12.8f%12.8f\n", $1*wave, $2*wave, $3*wave}' |\
    cat >! ${outmatrix}
    cat ${tempfile}.sg |\
    nawk -v SG=$SG '$3==SG{\
	    printf "%12.3f%12.3f%12.3f\n", 0, 0, 0;\
	    printf "%12.7f%12.7f%12.7f\n", 1, 0, 0;\
	    printf "%12.7f%12.7f%12.7f\n", 0, 1, 0;\
	    printf "%12.7f%12.7f%12.7f\n", 0, 0, 1;\
	    printf "%12.4f%12.4f%12.4f%12.4f%12.4f%12.4f\n", $15, $16, $17, $18, $19, $20;\
	    printf "%12.3f%12.3f%12.3f\n", 0, 0, 0; exit}' |\
    cat >> ${outmatrix}
    set CELL = `nawk 'NF==6{print;exit}' ${outmatrix}`
endif


echo "SYMM $SG"
echo "CELL $CELL"
echo "MATRIX ${outmatrix}"

# clean up
exit
rm -f peaks.file
rm -f PEAKS_WITH_PHIS
rm -f auto.om
rm -f ${tempfile}.log
rm -f ${tempfile}.sg
rm -f bravais.sum
rm -f b_to_m.in
rm -f b_to_m.out
rm -f analyse.out
exit
############################################################


Setup:

nawk 'BEGIN{exit}' >& /dev/null
if($status) alias nawk awk
set wrongSGs
set userSG
set cutoff = 0

# scan the command line for files
set i = 0
while( $i < $#argv )
    @ i = ( $i + 1 )
    @ nexti = ( $i + 1 )
    @ lasti = ( $i - 1 )
    if($nexti > $#argv) set nexti = $#argv
    if($lasti < 1) set lasti = 1
    set arg = "$argv[$i]"
    set ARG = `echo "$arg" | nawk '{print toupper($1)}'`
    
    if("$arg" =~ *.spt) then
	if(! -e "$arg") then
	    echo "WARNING: $arg does not exist! "
	    continue
	endif
	set spotfile = "$arg"
	
	continue
    endif
    
    if(-e "$arg") then
	# assume this is the mosflm.com file
	set temp = `nawk '/^WAVE/ && $2+0>0.1 && $2+0<10{print $2}' $arg`
	if("$temp" != "") set wavelength = "$temp"
	set temp = `nawk '/^DIST/ && $2+0>10 && $2+0<1000{print $2}' $arg`
	if("$temp" != "") set distance = "$temp"
	set temp = `nawk '/^TWOTHETA/ && $2+0>-90 && $2+0<90{print $2}' $arg`
	if("$temp" != "") set two_theta = "$temp"

	# get (and check) space group
	set temp = `nawk '/^SYMM/ && $2 ~ /^[PpCcIiFfRrHh][1-6]/{print $2}' $arg`
	set temp = `nawk -v SG=$temp '$4==SG{print $4;exit}' ${CLIBD}/symop.lib`
	if("$temp" != "") set SG = "$temp"

	# get unit cell
	set temp = `nawk '/CELL /{for(i=1;i<=NF;++i){if($i+0 > 10) printf $i+0 " "} print ""}' $arg |& tail -1`
	if($#temp == 6) then
	    foreach param ( $temp )
		# make sure entries are numbers
		if($param !~ [1-9]*) set temp = ""
	    end
	    if("$temp" != "")  set CELL = `echo $temp`
	endif

	# get upper resolution limit
	set temp = `nawk '$1 ~ /^RESO/{for(i=2;i<=NF;++i){print $i}}' $arg | nawk '$1+0>0.3 && $1+0 < 15' | tail -1`
	if($#temp == 1) then
	    if("$temp" =~ [0-9]*) set hiRES  = "$temp"
	endif
	continue
    endif
    
    # now read things that aren't files

    # check for space groups
    if("$ARG" =~ [PCIFHR][1-6]*) then
	set temp = "$ARG"
	if("$arg" =~ [PCIFHR][1-6]*) set temp = "$arg"
	set temp = `nawk -v SG=$temp '$4 == SG {print $4;exit}' $CLIBD/symop.lib`
	if("$temp" == "") then
	    # check for "pseudo-spacegroup" language
	    #set temp = `echo "$ARG" | nawk 'toupper($1) ~ /[PC]2212|[PC]2122|P21221|P22121/'`
	    # these are okay too, reindexing engine will understand
	endif
	if("$temp" != "") then
	    if($?NO) then
		# user doesn't want this space group
		set wrongSGs = ( $wrongSGs $temp )
		# check and see if $SG is in $wrongSGs
		foreach wrongSG ( $wrongSGs )
		    if("$SG" == "$wrongSG") set SG = ""
		end
		continue
	    endif
	    # user prefers this space group
	    set userSG = "$temp"
	    continue
	endif
    endif
    
    # allow disqualification of point groups
    if("$ARG" =~ PG[1-6]*) then
	# see if this is really a point group
	set temp = `nawk -v PG=$ARG '$5 == PG && ! /m/ && ! /bar/ {print $4;exit}' $CLIBD/symop.lib`
	if("$temp" != "") then
	    if($?NO) then
		# user doesn't want this point group
		set wrongSGs = ( $wrongSGs $ARG )
		
		# check and see if $SG is in $wrongSGs
		foreach wrongSG ( $temp )
		    if("$SG" == "$temp") set SG = ""
		end
		continue
	    endif
	    # user prefers this point group? 
	    set SG = "$temp[1]"
	    continue
	endif
    endif
    
    # allow disqualification of lattice types
    if(("$ARG" =~ *IC)||("$ARG" =~ *ONAL)) then
	# see if this is really a crystal system
	set temp = `nawk -v sys=$ARG '$6 == sys && ! /m/ && ! /bar/ {print $4;exit}' $CLIBD/symop.lib`
	if("$temp" != "") then
	    if($?NO) then
		# user doesn't want these space groups
		set wrongSGs = ( $wrongSGs $ARG )
		
		# check and see if $SG is in $wrongSGs
		foreach wrongSG ( $temp )
		    if("$SG" == "$wrongSG") set SG = ""
		end
		continue
	    endif
	    # user prefers this crystal system? 
	    set SG = "$temp[1]"
	    continue
	endif
    endif
    
    # allow disqualification of a bravais lattice
    if(("$arg" =~ [chotma][PICFR])) then
	# see if it's really a bravais lattice
	cat ${CLIBD}/symop.lib |\
	nawk -v brav=$arg 'BEGIN{sys=substr(brav,1,1);  type = substr(brav,2);\
	if(sys=="a") sys = "TRICL"; \
	if(sys=="m") sys = "MONOC"; \
	if(sys=="o") sys = "ORTHO"; \
	if(sys=="t") sys = "TETRA"; \
	if(sys=="h") sys = "TRIGO"; \
	if(sys=="c") sys = "CUBIC";}\
	$5 ~ /^PG/ && ! /m/ && ! /bar/ {\
	    SYS=substr($6,1,5); TYPE=substr($4,1,1);\
	    if(TYPE==type && (SYS==sys || (SYS=="HEXAG" && sys=="TRIGO"))){\
		    print $4}}' |\
	cat >! ${tempfile}brav
	set temp = `cat ${tempfile}brav`
	rm -f ${tempfile}brav >& /dev/null
	if("$temp" != "") then
	    if($?NO) then
		# user doesn't want these space groups
		set wrongSGs = ( $wrongSGs $arg )
		
		# check and see if $SG is in $wrongSGs
		foreach wrongSG ( $temp )
		    if("$SG" == "$wrongSG") set SG = ""
		end
		continue
	    endif
	    # user prefers this crystal system? 
	    set SG = "$temp[1]"
	    continue
	endif
    endif
    
    if("$arg" =~ [0-9]*) then
	# we have a number
	if(("$arg" =~ *A)||("$argv[$nexti]" == "A")) then
	    # user-preferred resolution limits
	    set temp = `echo "$arg" | nawk 'BEGIN{FS="-"} $1+0 > 0.1{print $1+0} $2+0 > 0.1{print $2+0}'`
	    if($#temp != 1) then
		set temp = `echo $temp | nawk '$1>$2{print $1, $2} $2>$1{print $2, $1}'`
		if($#temp == 2) then
		    # a range 3-10A is a resolution
		    set hiRES = `echo "$temp[2]" | nawk '{print $1+0}'`
		    set loRES = `echo "$temp[1]" | nawk '{print $1+0}'`
		    continue
		endif
	    else
		# single 1.1A is a wavelength
		set wavelength = `echo "$arg" | nawk '{print $1+0}'`
		continue
	    endif
	endif
	
	if(("$arg" =~ *mm)||("$argv[$nexti]" == "mm")) then
	    # distance is expressed in mm
	    set distance = `echo "$arg" | nawk '{print $1+0}'`
	    continue
	endif
	
	if(("$arg" =~ *sig)||("$argv[$nexti]" == "sig")) then
	    # sigma cutoff for spot use
	    set cutoff = `echo "$arg" | nawk '{print $1+0}'`
	    continue
	endif
	
	# interpret free integers as max cell edge?
	set int = `echo "$arg" | nawk '{printf "%d", $1+0}'`
	if("$arg" == "$int") then
	    set max_cell = $int
	endif
    endif
    
    
    # flags and switches
    if("$ARG" == "CELL") then
	if($?NO) set CELL = ""
	continue
    endif
    if("$ARG" == "SG") then
	if($?NO) set SG = ""
	continue
    endif

    unset NO
    if("$ARG" == NO) set NO
    if("$ARG" == NOT) set NO
end
# override any misc space-group settings with command-line space group
if("$userSG" != "") then
    set SG = "$userSG"
    set wrongSGs = ""
endif

# get the number of images that were peak-picked into this spotfile
set frames = `nawk '$NF=="-99.0" || $NF=="-999.0"' $spotfile | wc -l`
set spots = `nawk 'NF>5 && $1!="-99.00" && $1!="-999.00"' $spotfile | wc -l`

set target_sym
if("$SG" != "") then
    # set up target lattice/symm
    set target_sym = `nawk -v SG=$SG '$4==SG{print "target_sym", substr($4,1,1), $6;exit}' ${CLIBD}/symop.lib`
endif

goto Return_from_Setup
